local inventoryIndex = 0
local inventory = {}
local color = "|c8788EEffTradeFill:|r"
local log = "|c8788EEffTradeFill:|r Can't autoFill trade window with "

local TradeFill = LibStub("AceAddon-3.0"):GetAddon("TradeFill")

function tradePosition()
    for i = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(i) == nil then
            return i
        end
    end
end

function fillTradeWindow(item, position)

    if next(inventory) == nil then
        print(color .. " Item not found " .. item.link .. " |T" .. GetItemIcon(item.id) .. ":0|t")
        return
    end

    if position == nil then
        print(color .. " Trade window is full!|r")
        return
    else
        for i = 1, inventoryIndex do
            local locked = C_Container.GetContainerItemInfo(inventory[i].bag, inventory[i].slot)
            local stackCount = itemData(inventory[i].item)
            local stack = TradeFill.itemdb.profile.stack

            if inventory[i].bound == true then
                print(color .. " Item is Soulbound " .. item.link .. " |T" .. GetItemIcon(item.id) .. ":0|t can't trade that")
                return
            end

            if stack == false then

                if inventory[i].item == item.id and locked.isLocked == false then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end

                if i >= inventoryIndex then
                    print(color .. " Out of stock " .. item.link .. " |T" .. GetItemIcon(item.id) .. ":0|t no more stacks available")
                    return
                end
            end

            if stack == true then

                if inventory[i].item == item.id and locked.isLocked == false and inventory[i].count == stackCount.stack then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end

                if i >= inventoryIndex then
                    print(color .. " Out of stock " .. item.link .. " |T" .. GetItemIcon(item.id) .. ":0|t no more full stack available")
                    return
                end
            end
        end
    end
end

function ItemCondition(stack, level, guild, item)
    local guildName = GetGuildInfo("npc")

    if level == true and  UnitLevel("npc") < item.level then
        print(log .. item.link .. " |T" .. GetItemIcon(item.id) .. ":0|t not the required level")
        return
    elseif guild == true and guildName ~= GetGuildInfo("player") and GetGuildInfo("npc") ~= nil then
        print(log .. "|cFF00FF00<".. GetGuildInfo("npc") .. ">|r this not a guild member")
        return
    else
        for i = 1, stack do
            fillTradeWindow(item, tradePosition())
        end
    end
end

function tradingCondition(levelCap, stackA, stackB, playerNames, guildNames)
    local maxLevel = MAX_PLAYER_LEVEL_TABLE[GetAccountExpansionLevel()]

    if levelCap == true and UnitLevel("npc") ~= maxLevel then
        print(log .. "|cFF00FF00<".. UnitName("npc") .. ">|r player is not level " .. maxLevel)
        return true
    end

    if stackA == 0 and stackB == 0 then
        local _, ClassName, _ = UnitClass("npc")

        print(log .. "|cFF00FF00<".. capitalizeFirstLetter(ClassName) .. ">|r no stack number have been assigned")
        return true
    end

    for name in string.gmatch(playerNames, "[^\n]+") do
        if string.lower(name) == string.lower(GetUnitName("npc")) then
            print(log .. "|cFF00FF00<".. UnitName("npc") .. ">|r this player is on the exclude list")
            return true
        end
    end

    if GetGuildInfo("npc") then
        for guild in string.gmatch(guildNames, "[^\n]+") do
            if string.lower(guild) == string.lower(GetGuildInfo("npc")) then
                local guildName = GetGuildInfo("npc")

                print(log .. "|cFF00FF00<".. guildName .. ">|r this guild is on the exclude list")
                return true
            end
        end
    end
    return false
end

function groupCondition()

    if IsInRaid() then
        return { ["status"] = "raid" }
    elseif IsInGroup() then
        return { ["status"] = "party" }
    else
        return { ["status"] = "solo" }
    end
end

function TradeFill:TRADE_SHOW()
    local itemA = itemData(TradeFill.itemdb.profile.itemIdA)
    local itemB = itemData(TradeFill.itemdb.profile.itemIdB)
    local guild = TradeFill.itemdb.profile.guild
    local requireLevel = TradeFill.itemdb.profile.level

    local _, ClassName, _ = UnitClass("npc")

    local stackA = TradeFill.stacksdb.profile[ClassName].stackNumberA
    local stackB = TradeFill.stacksdb.profile[ClassName].stackNumberB

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                if bagItem.itemID == itemA.id or bagItem.itemID == itemB.id then
                    inventoryIndex = inventoryIndex + 1
                    inventory[inventoryIndex] = {
                        ["bag"] = bag,
                        ["slot"] = slot,
                        ["item"] = bagItem.itemID,
                        ["bound"] = bagItem.isBound,
                        ["count"] = bagItem.stackCount,
                        ["link"] = bagItem.hyperlink
                    }
                end
            end
        end
    end

    local group = groupCondition()

    local stranger = TradeFill.itemdb.profile.solo
    local party = TradeFill.itemdb.profile.party
    local raid = TradeFill.itemdb.profile.raid

    local players = TradeFill.itemdb.profile.players
    local guilds = TradeFill.itemdb.profile.guilds
    local levelCap = TradeFill.itemdb.profile.max

    if stranger == true and group.status == "solo" then

        if tradingCondition(levelCap, stackA, stackB, players, guilds) then
            return
        end

        if itemA.id > 0 and stackA > 0 then
            ItemCondition(stackA, requireLevel, guild, itemA)
        end

        if itemB.id > 0 and stackB > 0 then
            ItemCondition(stackB, requireLevel, guild, itemB)
        end
    end

    if (party == true and group.status == "party") or (raid == true and group.status == "raid") then

        if tradingCondition(levelCap, stackA, stackB, players, guilds) then
            return
        end

        local numGroupMembers = GetNumGroupMembers()

        if numGroupMembers > 0 then
            for i = 1, numGroupMembers do
                local unitName = UnitName(group.status .. i)

                if unitName == UnitName("npc") then

                    if itemA.id > 0 and stackA > 0 then
                        ItemCondition(stackA, requireLevel, guild, itemA)
                    end

                    if itemB.id > 0 and stackB > 0 then
                        ItemCondition(stackB, requireLevel, guild, itemB)
                    end
                    break
                end
            end
        return
        end
    end
end

TradeFill:RegisterEvent("TRADE_SHOW")

function TradeFill:TRADE_CLOSED()
    inventoryIndex = 0
    inventory = {}
end

TradeFill:RegisterEvent("TRADE_CLOSED")